package net.knarcraft.hungerarena;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.HashMap;

public class SpawnsCommand implements CommandExecutor {

    public final HungerArena plugin;
    int tributeId = 0;
    int arenaId = 0;
    boolean noPlayerSpawns;

    public SpawnsCommand(HungerArena plugin) {
        this.plugin = plugin;
    }

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String commandLabel,
                             String[] args) {
        Player player = (Player) sender;
        String playerWorld = player.getWorld().getName();
        this.noPlayerSpawns = true;
        for (int arenaId : plugin.arenas.keySet()) {
            String arenaWorld = plugin.worldNames.get(arenaId);
            if (arenaWorld != null) {
                if (arenaWorld.equals(playerWorld)) {
                    this.arenaId = arenaId;
                    this.noPlayerSpawns = false;
                    break;
                }
            }
        }

        if (!command.getName().equalsIgnoreCase("StartPoint")) {
            return false;
        }

        if (!player.hasPermission("HungerArena.StartPoint")) {
            player.sendMessage(ChatColor.RED + "You don't have permission!");
            return false;
        }

        if (args.length >= 2) {
            try {
                this.tributeId = Integer.parseInt(args[1]);
                this.arenaId = Integer.parseInt(args[0]);
            } catch (Exception e) {
                player.sendMessage(ChatColor.RED + "Argument not an integer!");
                return true;
            }
        }

        if (args.length == 6) {
            saveGivenLocationSpawnPoint(args, player);
            return true;
        } else if (args.length >= 2) {
            savePlayerLocationSpawnPoint(player);
            return true;
        } else if (args.length == 1) {
            enableSpawnSetting(args, player);
            return true;
        } else {
            player.sendMessage(ChatColor.RED + "No argument given! \nUse command like this:\n/startpoint [Arena #] " +
                    "[Startpoint #] for setting your position as a startpoint.\n/startpoint [Arena #] [Startpoint #] " +
                    "[Mapname] [x] [y] [z] \nOr you can use /startpoint [Arena #] to use the 'spawntool': ID" +
                    plugin.config.getInt("spawnsTool") + " for setting the startpoints!");
            return false;
        }
    }

    private void enableSpawnSetting(String[] args, Player player) {
        //If any player spawn has been set, the arena id was set during the constructor call
        if (noPlayerSpawns) {
            try {
                this.arenaId = Integer.parseInt(args[0]);
            } catch (Exception e) {
                player.sendMessage(ChatColor.RED + "Argument not an integer!");
                return;
            }
        }

        //If spawns already exist, calculate the next missing spawn
        int start = 1;
        if (plugin.spawns.get("Spawns." + arenaId) != null) {
            while (start < plugin.config.getInt("maxPlayers") + 2) {
                if (plugin.spawns.get("Spawns." + arenaId + "" + start) != null) {
                    start++;
                    //All spawn points already set
                    if (start == plugin.config.getInt("maxPlayers") + 1) {
                        player.sendMessage(ChatColor.DARK_AQUA + "[HungerArena] " + ChatColor.GREEN +
                                "All spawns set, type /startpoint [Arena #] [Spawn #] to over-ride previous points!");
                        return;
                    }
                } else {
                    //Enable spawn setting, starting from the next available spawn
                    player.sendMessage(ChatColor.DARK_AQUA + "[HungerArena] " + ChatColor.RED +
                            "Begin Setting For Arena " + arenaId + " Starting From Point " + start);
                    plugin.setting.put(player.getName(), arenaId + "-" + start);
                    return;
                }
            }
        }
        //Enable spawn setting, starting from the next available spawn
        player.sendMessage(ChatColor.DARK_AQUA + "[HungerArena] " + ChatColor.RED + "Begin Setting For Arena " +
                arenaId + " Starting From Point " + start);
        plugin.setting.put(player.getName(), arenaId + "-" + start);
    }

    private void savePlayerLocationSpawnPoint(Player player) {
        if (tributeId >= 1 && tributeId <= plugin.config.getInt("maxPlayers")) {
            player.sendMessage(ChatColor.RED + "You can't go past " + plugin.config.getInt("maxPlayers") + " players!");
            return;
        }

        if (!plugin.worldNames.containsValue(player.getWorld().getName())) {
            player.sendMessage(ChatColor.GOLD + "You've added this world to the config ...");
        }
        Location location = player.getLocation().getBlock().getLocation();
        double x = location.getX() + .5;
        double y = location.getY();
        double z = location.getZ() + .5;
        location = new Location(location.getWorld(), x, y, z);

        saveSpawnPoint(player, location);
    }

    private void saveGivenLocationSpawnPoint(String[] args, Player player) {
        String worldName = args[2];
        double x = Double.parseDouble(args[3]);
        double y = Double.parseDouble(args[4]);
        double z = Double.parseDouble(args[5]);
        Location location = new Location(Bukkit.getWorld(worldName), x, y, z);

        saveSpawnPoint(player, location);
    }

    private void saveSpawnPoint(Player player, Location location) {
        //Add a new arena if it does not yet exist
        if (plugin.arenas.get(arenaId) == null) {
            plugin.arenas.put(arenaId, new Arena(arenaId, location.getWorld(), new ArrayList<>(),
                    ArenaState.IDLE, false, true, 0, 0, 0,
                    new HashMap<>()));
        }
        Arena arena = plugin.arenas.get(arenaId);
        arena.getLocations().put(tributeId, location);

        World world = location.getWorld();
        String coordinates = world == null ? "" : world.getName() + "," + location.getX() + "," +
                location.getY() + "," + location.getZ();
        player.sendMessage(coordinates);
        plugin.spawns.set(String.format("Spawns.%s.%s", arenaId, tributeId), coordinates);
        plugin.worldNames.put(arenaId, location.getWorld().getName());
        plugin.saveSpawns();

        arena.setMaxPlayers(plugin.location.get(arenaId).size());
        player.sendMessage(ChatColor.AQUA + String.format("You have set the spawn location of Tribute %d in arena %d!", tributeId, arenaId));
        this.plugin.reloadSpawnPoints(false);
    }

}
