package net.knarcraft.hungerarena;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

public class SponsorCommands implements CommandExecutor {
    public final HungerArena plugin;

    public SponsorCommands(HungerArena m) {
        this.plugin = m;
    }

    @Override
    public boolean onCommand(@NotNull CommandSender sender, Command cmd, @NotNull String commandLabel, String[] args) {
        if (cmd.getName().equalsIgnoreCase("Sponsor")) {
            if (sender instanceof Player p) {
                if (p.hasPermission("HungerArena.Sponsor")) {
                    if (plugin.getArena(p) == null) {
                        if (args.length == 0) {
                            p.sendMessage(ChatColor.RED + "You didn't specify a tribute!");
                            return false;
                        }
                        if (args.length == 1) {
                            p.sendMessage(ChatColor.RED + "You didn't specify an item!");
                        }
                        if (args.length == 2) {
                            p.sendMessage(ChatColor.RED + "You didn't specify an amount!");
                        }
                        if (args.length >= 3) {
                            Player target = Bukkit.getServer().getPlayer(args[0]);
                            if (target == null || plugin.getArena(target) == null) {
                                p.sendMessage(ChatColor.RED + "That person isn't playing!");
                            } else {
                                try {
                                    String ID = args[1].toUpperCase().trim();
                                    int amount = Integer.parseInt(args[2]);
                                    if ((!plugin.management.getStringList("sponsors.blacklist").isEmpty() && !plugin.management.getStringList("sponsors.blacklist").contains(ID)) || plugin.management.getStringList("sponsors.blacklist").isEmpty()) {
                                        handleItemsAndEco(p, args, ID, amount, target);
                                    } else {
                                        p.sendMessage(ChatColor.RED + "You can't sponsor that item!");
                                        p.sendMessage(ChatColor.GREEN + "Other items you can't sponsor are:");
                                        for (String blacklist : plugin.management.getStringList("sponsors.blacklist")) {
                                            p.sendMessage(ChatColor.AQUA + blacklist);
                                        }
                                    }
                                } catch (Exception e) {
                                    p.sendMessage(ChatColor.RED + "Something went wrong there... Make sure that you do like this /sponsor [name] [item] [number]");
                                }
                            }
                        }
                    } else {
                        p.sendMessage(ChatColor.RED + "You are playing, you can't sponsor yourself!");
                    }
                } else {
                    p.sendMessage(ChatColor.RED + "You don't have permission!");
                }
            } else if (sender instanceof ConsoleCommandSender) {
                if (args.length == 0) {
                    sender.sendMessage(ChatColor.RED + "You didn't specify a tribute!");
                    return false;
                }
                if (args.length == 1) {
                    sender.sendMessage(ChatColor.RED + "You didn't specify an item!");
                }
                if (args.length == 2) {
                    sender.sendMessage(ChatColor.RED + "You didn't specify an amount!");
                }
                if (args.length >= 3) {
                    Player target = Bukkit.getPlayer(args[0]);
                    if (target == null) {
                        throw new NullPointerException("getPlayer returned null");
                    }
                    String materialId = args[1].toUpperCase().trim();
                    int materialAmount = Integer.parseInt(args[2]);
                    try {
                        if ((!plugin.management.getStringList("sponsors.blacklist").isEmpty() && !plugin.management.getStringList("sponsors.blacklist").contains(materialId)) || plugin.management.getStringList("sponsors.blacklist").isEmpty()) {
                            Material material = Material.getMaterial(materialId);
                            if (material == null) {
                                throw new NullPointerException("getMaterial returned null");
                            }
                            ItemStack sponsorItem = new ItemStack(material, materialAmount);
                            if (plugin.getArena(target) == null) {
                                sender.sendMessage(ChatColor.RED + "That person isn't playing!");
                            } else {
                                sender.sendMessage(ChatColor.RED + "You can't sponsor yourself!");
                                target.sendMessage(ChatColor.AQUA + "You have been Sponsored!");
                                target.getInventory().addItem(sponsorItem);
                                sender.sendMessage("You have sponsored " + target.getName() + "!");
                            }
                        } else {
                            sender.sendMessage(ChatColor.RED + "You can't sponsor that item!");
                            sender.sendMessage(ChatColor.GREEN + "Other items you can't sponsor are:");
                            for (String blacklist : plugin.management.getStringList("sponsors.blacklist")) {
                                sender.sendMessage(ChatColor.AQUA + blacklist);
                            }
                        }
                    } catch (Exception e) {
                        sender.sendMessage(ChatColor.RED + "Something went wrong there... Make sure that you do like this /sponsor [name] [material] [number]");
                    }
                }
            }
        }
        return false;
    }

    private void handleItemsAndEco(Player player, String[] args, String materialId, int amount, Player target) {
        Material sponsorMaterial = plugin.getNewMaterial(materialId, 0);
        ItemStack sponsorItem;
        boolean done = false;
        if (sponsorMaterial != null) {
            sponsorItem = new ItemStack(sponsorMaterial, amount);
        } else {
            player.sendMessage(ChatColor.RED + "That item does not exist !!");
            return;
        }
        for (ItemStack Costs : plugin.Cost) {
            if (!player.getInventory().containsAtLeast(Costs, Costs.getAmount() * amount)) {
                player.sendMessage(ChatColor.RED + "You don't have the necessary items to sponsor!");
                return;
            }
        }
        if (args[0].equalsIgnoreCase(player.getName())) {
            player.sendMessage(ChatColor.RED + "You can't sponsor yourself!");
        } else {
            if (plugin.config.getBoolean("sponsorEco.enabled")) {
                if (!(plugin.econ.getBalance(player) < (plugin.config.getDouble("sponsorEco.cost") * amount))) {
                    plugin.econ.withdrawPlayer(player, plugin.config.getDouble("sponsorEco.cost") * amount);
                    done = true;
                } else {
                    player.sendMessage(ChatColor.RED + "You don't have enough money to do that!");
                    return;
                }
            }
            if (!plugin.Cost.isEmpty()) {
                for (ItemStack aCosts : plugin.Cost) {
                    for (int x = 1; x <= amount; x++) {
                        player.getInventory().removeItem(aCosts);
                        done = true;
                    }
                }
            }
            if (done) {
                target.getInventory().addItem(sponsorItem);
                target.sendMessage(ChatColor.AQUA + "You have been Sponsored!");
                player.sendMessage("You have sponsored " + target.getName() + "!");
            } else {
                player.sendMessage(ChatColor.RED + "Sponsoring is disabled!");
            }
        }
    }

}
